<?
  // net.php

  // Toutes les fonctions touchant le reseau

  error_reporting(E_ALL | E_STRICT);

  ////////////////////////
  // TESTS DE DEMARRAGE //
  ////////////////////////

  ///////////////
  // FONCTIONS //
  ///////////////
  function gethostbyaddr_timeout($ip, $server_dns, $nb_secondes=3)
  {
    // Envoie une requete de test DNS a un serveur. La fonction repond
    // rc==0 -> OK  (le serveur repond et sait resoudre l'adresse IP)
    // rc=-1 -> BAD (le serveur n'a pas repondu dans les $nb_secondes)
    // rc=-2 -> BAD (le serveur repond mais ne sait pas resoudre l'adresse IP)
    // rc=-3 -> BAD (l'adresse fournie n'a pas 4 octets)
    // Parametres demandes :
    // $ip          : adresse IP a faire resoudre par le DNS
    // $server_dns  : adresse du serveur DNS
    // $nb_secondes : temps de reponse avant timeout

    // random transaction number (for routers etc to get the reply back)
    $data = rand(10, 99);
    // trim it to 2 bytes
    $data = substr($data, 0, 2);
    // request header
    $data .= "\1\0\0\1\0\0\0\0\0\0";
    // split IP up
    $bits = explode(".", $ip);
    // error checking #echo "ERROR: pas 4 octets a l'adresse";
    if (count($bits) != 4) return -3;
    // loop through each segment
    for ($x=3; $x>=0; $x--)
    {
      // needs a byte to indicate the length of each segment of the request
      switch (strlen($bits[$x]))
      {
        case 1: // 1 byte long segment
          $data .= "\1"; break;
        case 2: // 2 byte long segment
          $data .= "\2"; break;
        case 3: // 3 byte long segment
          $data .= "\3"; break;
        default: // segment is too big, invalid IP
          return -3; #echo "INVALID";
      }
      // and the segment itself
      $data .= $bits[$x];
    }
    // and the final bit of the request
    $data .= "\7in-addr\4arpa\0\0\x0C\0\1";
    // create UDP socket
    $handle = @fsockopen("udp://$server_dns", 53, $errno, $errstr,
                         $nb_secondes);
    if (!$handle)
      return -1;
    // send our request (and store request size so we can cheat later)
    $requestsize = fwrite($handle, $data);
    $Type = unpack("s", $data);
    socket_set_timeout($handle, $nb_secondes);
    // hope we get a reply
    $response = fread($handle, 1000);
    fclose($handle);
    if ($response == "") {
      return -1; // le serveur DNS n'a pas repondu apres $nb_secondes
    }
    // find the response type
    $type = unpack("s", substr($response, $requestsize+2));
    if ($type[1] == 0x0C00)  // answer
    {
      // OK, le serveur repond
      // set up our variables
      $host="";
      $len = 0;
      // set our pointer at the beginning of the hostname
      // uses the request size from earlier rather than work it out
      $position=$requestsize+12;
      // reconstruct hostname
      do # while ci-dessous
      {
        // get segment size
        // decodage caractere par caractere
        $len = unpack("c", substr($response, $position));
        // null terminated string, so length 0 = finished
        if ($len[1] == 0)
        {
          // return the hostname, without the trailing .
          // return substr($host, 0, strlen($host) -1);
          // Comme on est dans un test de DNS, on renvoie 0 car le serveur a
          // repondu
          return 0;
        }
        // add segment to our host
        $host .= substr($response, $position+1, $len[1]) . ".";
        // move pointer on to the next segment
        $position += $len[1] + 1;
      }
      while ($len != 0);
      // error - return the hostname we constructed (without the . on the end)
      return -2; // Le serveur repond une requete invalide
    }
    return -2; // Le serveur repond mais ne sait pas resoudre l'adresse
  }

  function testNFSv4TCP ($serveur, $port, $nb_secondes)
  {
    // port par defaut 2049
    // Cette fonction verifie si le serveur NFS est bien lance sur le $serveur
    // Utilise le port passe (devrait etre 2049).
    // Renvoie TRUE si le serveur est UP et repond a l'authentification
    // Renvoie FALSE si le serveur est DOWN ou ne veut pas communiquer.
    $data = pack ("CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC",
      0x80, 0x00, 0x00, 0x28, // Fragment Header 40octets
      0x5a, 0x2e, 0x03, 0x53, // XID : Nombre aleatoire
      0x00, 0x00, 0x00, 0x00, // Call : 0
      0x00, 0x00, 0x00, 0x02, // RPC version 2
      0x00, 0x01, 0x86, 0xA3, // NFS : 100003 (186A3 en Hexa)
      0x00, 0x00, 0x00, 0x04, // NFS version 4
      0x00, 0x00, 0x00, 0x00, // Procedure NULL
      0x00, 0x00, 0x00, 0x00, // Credential AUTH_NULL
      0x00, 0x00, 0x00, 0x00, // LENGTH = 0
      0x00, 0x00, 0x00, 0x00, // Verifier AUTH NULL
      0x00, 0x00, 0x00, 0x00  // LENGTH = 0
    );
    // create TCP socket
    $handle = @fsockopen("tcp://$serveur", $port, $errno, $errstr,
                         $nb_secondes);
    if (!$handle)
      return FALSE;
    socket_set_timeout($handle, $nb_secondes);

    // send our request (and store request size so we can cheat later)
    $requestsize = fwrite($handle, $data);


    // hope we get a reply
    $response = fread($handle, 1000);
    fclose($handle);
    if ($response == "") {
      return -1; // le serveur NFS n'a pas repondu apres $nb_secondes
    }
    //trigger_error ("Reponse du serveurNFS : $response", E_USER_NOTICE);
    return TRUE;
  }

  function testPortmapTCP ($serveur, $port, $nb_secondes)
  {
    $data = pack ("CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC", 
      0x80, 0x00, 0x00, 0x28, // Fragment Header 40octets
      0x5a, 0x2e, 0x03, 0x53, // XID : Nombre aleatoire
      0x00, 0x00, 0x00, 0x00, // Call : 0
      0x00, 0x00, 0x00, 0x02, // RPC version 2
      0x00, 0x01, 0x86, 0xA0, // Portmap : 100000 (186A0 en Hexa)
      0x00, 0x00, 0x00, 0x04, // Portmap version 2
      0x00, 0x00, 0x00, 0x04, // Procedure DUMP
      0x00, 0x00, 0x00, 0x00, // Credential AUTH_NULL
      0x00, 0x00, 0x00, 0x00, // LENGTH = 0
      0x00, 0x00, 0x00, 0x00, // Verifier AUTH NULL
      0x00, 0x00, 0x00, 0x00  // LENGTH = 0
    );
    // create TCP socket
    $handle = @fsockopen("tcp://$serveur", $port, $errno, $errstr,
                         $nb_secondes);
    if (!$handle)
      return FALSE;
    socket_set_timeout($handle, $nb_secondes);

    // send our request (and store request size so we can cheat later)
    $requestsize = fwrite($handle, $data);


    // hope we get a reply
    $response = fread($handle, 1000);
    fclose($handle);
    if ($response == "") {
      return -1; // le serveur Portmap n'a pas repondu apres $nb_secondes
    }
    trigger_error ("Reponse du serveurPortmap : $response", E_USER_NOTICE);
    return TRUE;
  }

  ////////////////////////////
  // FONCTIONS POUR LE PING //
  ////////////////////////////
  function __ping_ip_checksum($data)
  {
     $sum=0;
     for($i=0;$i<strlen($data);$i += 2)
     {
         if($data[$i+1]) $bits = unpack('n*',$data[$i].$data[$i+1]);
         else $bits = unpack('C*',$data[$i]);
         $sum += $bits[1];
     }

     while ($sum>>16) $sum = ($sum & 0xffff) + ($sum >> 16);
     $checksum = pack('n1',~$sum);
     return $checksum;
  }

  function __ping_start_time()
  {
    $__ping_timer_start_time = microtime();
    return $__ping_timer_start_time;
  }

  function __ping_get_time($__ping_timer_start_time, $acc=2)
  {
    // format start time
    $start_time = explode (" ", $__ping_timer_start_time);
    $start_time = $start_time[1] + $start_time[0];
    // get and format end time
    $end_time = explode (" ", microtime());
    $end_time = $end_time[1] + $end_time[0];
    return number_format ($end_time - $start_time, $acc);
  }

  function __ping_Build_Packet()
  {
    global $__ping_request,$__ping_request_len;
    $data = "abcdefghijklmnopqrstuvwabcdefghi"; // the actual test data
    $type = "\x08"; // 8 echo message; 0 echo reply message
    $code = "\x00"; // always 0 for this program
    $chksm = "\x00\x00"; // generate checksum for icmp request
    $id = "\x00\x00"; // we will have to work with this later
    $sqn = "\x00\x00"; // we will have to work with this later

    // now we need to change the checksum to the real checksum
    $chksm = __ping_ip_checksum($type.$code.$chksm.$id.$sqn.$data);

    // now lets build the actual icmp packet
    $__ping_request = $type.$code.$chksm.$id.$sqn.$data;
    $__ping_request_len = strlen($__ping_request);
  }

  function ping($dst_addr, &$__ping_errstr, $timeout=5, $percision=3,
                $src_addr="", $broadcast="")
  {
    if (posix_getuid()!=0)
      die ("Ce programme doit être lancé en tant qu'utilisateur root\n");
    // On initialise, sinon on peut avoir des retours bizarres
    $__ping_errstr = "OK";
    $__ping_icmp_socket = socket_create (AF_INET, SOCK_RAW, 1);
    socket_set_block ($__ping_icmp_socket);
    // lets catch dumb people
    if ((int) $timeout <= 0) $timeout = 5;
    if ((int) $percision <= 0) $percision = 3;

    // set the timeout
    socket_set_option ($__ping_icmp_socket,
      SOL_SOCKET,  // socket level
      SO_RCVTIMEO, // timeout option
      array(
        "sec"=>$timeout, // Timeout in seconds
        "usec"=>0  // I assume timeout in microseconds
      )
    );

    if ($dst_addr)
    {
      // Si on fournit l'adresse source, on la pose ici
      if ($src_addr != "")
      {
        socket_bind ($__ping_icmp_socket, $src_addr);
      }

      // Si on envoie un broadcast, il faut demander que la socket soit de type
      // broadcast
      if ($broadcast != "")
      {
        socket_set_option($__ping_icmp_socket, SOL_SOCKET, SO_BROADCAST, 1);
      }

      // On envoie le paquet ICMP
      if (@socket_connect ($__ping_icmp_socket, $dst_addr, NULL))
      {

      }
      else
      {
        $__ping_errstr = "Cannot connect to $dst_addr";
        return FALSE;
      }
      global $__ping_request;
      global $__ping_request_len;
      __ping_Build_Packet ();
      $__ping_timer_start_time = __ping_start_time ();
      socket_write ($__ping_icmp_socket, $__ping_request, $__ping_request_len);
      $__ping_reply = "";
      if ($broadcast != "")
      {
        // Les broadcasts ne repondent pas : on n'attend pas la reponse
        return 0;
      }
      if (socket_recv ($__ping_icmp_socket, &$__ping_reply, 256, 0))
      {
        $__ping_time = __ping_get_time ($__ping_timer_start_time, $percision);
        return $__ping_time;
      }
      else
      {
        $__ping_errstr = "Time out";
        return FALSE;
      }
    }
    else
    {
      $__ping_errstr = "Destination address not specified";
      return FALSE;
    }
  }

  function pingUp ($machine, $timeout, $essais)
  {
    // Cette fonction verifie si la machine passee en argument est active en
    // faisant une serie de ping. On donne a chaque ping $timeout pour repondre.
    // Apres $essais, la machine est declaree en panne.
    // Renvoie TRUE si la machine est Up.
    // Renvoie FALSE si la machine est Down.
    if (!function_exists("criclog"))
      die ("La librairie logger n'est pas chargée dans net\n");

    if ($timeout<1)
    {
      criclog ("ERREUR : pingUp avec un timeout <1 ($timeout) : DEHORS");
      return TRUE;
    }
    if ($essais<1)
    {
      criclog ("ERREUR : pingUp avec un nombre d'essais <1 ($essais) : DEHORS");
      return TRUE;
    }
    for ($i=0 ; $i<$essais ; $i++)
    {
      $ping = ping ($machine, $ping_errstr, $timeout);
      if ($ping === FALSE || $ping_errstr === "Time Out")
      {
        // On ne joint pas la machine, on incremente $i
        // Des fois, ping renvoie TimeOut mais avec une valeur de $ping >0
        criclog ("Pas de réponse de $machine en ping (".($i+1)."/".$essais.")");
        if ($ping_errstr != "Time Out")
        {
          // Si on n'a pas eu le Timeout, on attend ici les secondes
          sleep ($timeout);
        }
      }
      else
      {
        // Ouf, on joint $machine : on sort du test, car je suis le
        // seul serveur autorise
        trigger_error ("On joint $machine : on continue", E_USER_NOTICE);
        // Si le compteur n'est pas a 0, cela veut dire que le test d'avant
        // a echoue, mais que maintenant la connexion fonctionne. On met
        // dans le log une indication de retour a la normale
        if ($i > 0)
        {
          criclog ("On joint à nouveau $machine en ping");
        }
        $i = 0;
        return TRUE;
      }
    }
    if ($i >= $essais)
    {
      criclog ("On ne joint pas $machine en ping");
      return FALSE;
    }
    criclog ("ERREUR : Passage à la fin de pingUP INTERDIT !!!");
    return TRUE;
  }

  function ipOuNom($adresse)
  {
    // Cette fonction renvoie l'adresse IP ou le nom contenu dans $adresse.
    // $adresse peut etre de la forme :
    // 1.2.3.4 ou 1.2.3.4:1111 ou mac.domain.tld ou mac.domain.tld:1111
    if (($pos=strpos($adresse, ":"))!==FALSE)
    {
      return substr($adresse, 0, $pos);
    }
    return $adresse;
  }

  function testTCP($host, $port, $test, $ssl, $waitPrompt, $timeout=5)
  {
    // Cette fonction envoie le test $test a la machine $host sur le port $port,
    // en utilisant la bibliotheque ssl si necessaire. Si le $waitPrompt est
    // actif, on attend le prompt avant d'envoyer le test.
    // Renvoie TRUE si le service fonctionne correctement
    // Renvoie FALSE si le service ne fonctionne pas du tout ou pas dans le
    // temps imparti
    // $timeout=5; // Temps avant de declarer le service HS
    trigger_error("Entree dans testTCP($host, $port, $test, $ssl, $waitPrompt)",
                  E_USER_NOTICE);
    $transport="tcp";
    trigger_error("$transport://$host:$port", E_USER_NOTICE);
    $fp=@stream_socket_client("$transport://$host:$port", $errno, $errstr,
                              $timeout);
    if (!$fp)
    {
      trigger_error ("FERME : $transport://$host:$port -> $errstr ($errno)",
               E_USER_WARNING);
      return FALSE;
    }
    if ($ssl!=0)
    {
      trigger_error("Passage en SSL", E_USER_NOTICE);
      stream_socket_enable_crypto($fp, TRUE, 
                                  STREAM_CRYPTO_METHOD_SSLv23_CLIENT);
    }

    stream_set_timeout($fp, $timeout);
    // On attend le prompt, si le $waitPrompt est actif
    if ($waitPrompt)
    {
      $ligne=trim(fgets($fp, 1024));
      trigger_error("< $ligne", E_USER_NOTICE);
      $info = stream_get_meta_data($fp);
      if ($info['timed_out'])
      {
        trigger_error ("FERME : Connection timed out avant la banniere !",
                       E_USER_WARNING);
        fclose($fp);
        return FALSE;
      }
    }
    // On envoie le test :
    trigger_error("> $test", E_USER_NOTICE);
    // Bien laisser les retours-chariots ici sinon ils ne passent pas dans les
    // parametres de la fonction
    fwrite($fp, "$test\r\n");
    // On recupere la sortie
    $result=FALSE;
    while (($ligne=fgets($fp, 1024))!==FALSE)
    {
      $result.=trim($ligne);
    }
    $info = stream_get_meta_data($fp);
    if ($info['timed_out'])
    {
      trigger_error ("FERME : Connection timed out!", E_USER_WARNING);
      fclose($fp);
      trigger_error ("< $result", E_USER_NOTICE);
      return FALSE;
    }

    fclose($fp);
    trigger_error ("< $result", E_USER_NOTICE);
    trigger_error("$transport://$host:$port : Connexion OK", E_USER_WARNING);
    return TRUE;
  }

  function listeInterfaces()
  {
    // Cette fonction renvoie la liste de toutes les interfaces actuellement
    // operationnelles sur cette machine dans un tableau
    // Renvoie FALSE si le fichier /sbin/ip n'existe pas

    $filePoint = popen ("/sbin/ip addr show", "r");
    if ($filePoint === FALSE)
      return FALSE;
    $arrInterface = array();
    // On va chercher dans ip show addr toutes les lignes qui ont inet (ipv4).
    // On recupere l'interface qui est la derniere partie de la ligne
    // Ex : inet 10.10.10.12/32 scope global lo:0
    while (($ligne = fgets ($filePoint, 1024)) !== FALSE)
    {
      if (strpos ($ligne, "inet ") !== FALSE)
      {
        $arrInterface[] = ltrim (strrchr (trim ($ligne), " "));
      }
    }
    pclose ($filePoint);
    return ($arrInterface);
  }

  function interfaceReelle ($interface)
  {
    // Cette fonction renvoie l'interface reelle associee a l'interface passee
    // Ex. On envoie lo:0, renvoie lo.
    // Ex. On envoie lo, renvoie lo (c'est deja une interface reelle)
    // Ne verifie pas si l'interface existe
    if (($pos = strpos ($interface, ":")) !== FALSE)
    {
      return substr ($interface, 0, $pos);
    }
    return $interface;
  }

  function listeIP()
  {
    // Cette fonction renvoie un tableau contenant la liste des IP associees a
    // chaque interface. La cle est l'IP, la valeur est l'interface et le cidr,
    // car il peut y avoir plusieurs IP par interface, mais qu'une seule IP
    // identique.
    // Renvoie FALSE en cas de probleme
    if (!function_exists("criclog"))
      die ("La librairie logger n'est pas chargée dans net\n");
    $filePoint = popen ("/sbin/ip addr show", "r");
    if ($filePoint === FALSE)
    {
      return FALSE;
    }
    $arrInterface = FALSE;
    while (($ligne = fgets ($filePoint, 1024)) !== FALSE)
    {
      if (strpos ($ligne, "inet ") !== FALSE)
      {
        $ligne = trim ($ligne);
        $interfaceLue = ltrim (strrchr ($ligne, " "));
        list ($inet, $ipCidr, $suite) = explode (" ", $ligne);
        // On va supprimer le cidr pour l'adresse IP lue (si il existe).
        // Il devrait toujours exister, mais, sait-on jamais.
        if (($pos = strpos ($ipCidr, "/")) !== FALSE)
        {
          $ip = substr ($ipCidr, 0, $pos);
          $cidr = substr ($ipCidr, $pos + 1);
          $arrInterface[$ip]["interface"] = $interfaceLue;
          $arrInterface[$ip]["cidr"] = $cidr;
        }
        else
        {
          criclog ("ERREUR : $interfaceLue n'a pas de CIDR dans listeIP");
        }
      }
    }
    return $arrInterface;
  }

  function ipActiveeInterface ($interface, $IPActive)
  {
    // Verifie si l'interface passee en argument a l'adresse IP proposee.
    // Renvoie TRUE si l'IP est activee
    // Renvoie FALSE si l'IP est absente
    $filePoint = popen ("/sbin/ip addr show", "r");
    if ($filePoint === FALSE)
    {
      return FALSE;
    }
    $arrInterface = array();
    // On va chercher dans ip show addr toutes les lignes qui ont inet (ipv4).
    // On recupere l'interface qui est la derniere partie de la ligne
    // Ex : inet 10.10.10.12/32 scope global lo:0
    while (($ligne = fgets ($filePoint, 1024)) !== FALSE)
    {
      if (strpos ($ligne, "inet ") !== FALSE)
      {
        $ligne = trim ($ligne);
        $interfaceLue = ltrim (strrchr ($ligne, " "));
        if ($interfaceLue == $interface)
        {
          list ($inet, $ip, $suite) = explode (" ", $ligne);
          // On va supprimer le cidr pour l'adresse IP lue (si il existe).
          // Il devrait toujours exister, mais, sait-on jamais.
          if (($pos = strpos ($ip, "/")) !== FALSE)
          {
            $ip = substr ($ip, 0, $pos);
          }
          if ($ip === $IPActive)
          {
            trigger_error ("ipVirtuelleActiveeInterface : $IPActive sur ".
                           "$interface : OK", E_USER_NOTICE);
            pclose ($filePoint);
            return (TRUE);
          }
        }
      }
    }
    trigger_error ("ipVirtuelleActiveeInterface : $IPActive sur $interface ".
                   ": BAD", E_USER_NOTICE);
    pclose ($filePoint);
    return (FALSE);
  }

  function cidr2netmask ($cidr)
  {
    // Cette fonction renvoie le netmask en fonction du cidr passe en argument
    // Ex. Renvoie 255.255.255.0 pour un cidr en 24.
    return long2ip(~((1 << (32 - $cidr)) - 1));
  }

  function ipInNetwork ($ip, $network, $cidr)
  {
    // Cette fonction verifie si l'adresse IP passee est bien dans le reseau
    // $network, masque par le cidr
    $ip = ip2long ($ip);
    $network = ip2long ($network);
    if ($cidr == 0)
      $netmask = 0;
    else
      $netmask = (~((1 << (32 - $cidr)) - 1));
    if (($ip & $netmask) === ($network & $netmask))
    {
      return TRUE;
    }
    return FALSE;
  }

  function netmask2cidr ($netmask)
  {
    // Cette fonction renvoie le cidr associe au netmask passe en argument
    // Ex. Renvoie 24 pour un masque de 255.255.255.0
    // Renvoie FALSE si l'IP passee n'est pas valide
    $netmask = ip2long ($netmask);
    if ($netmask === FALSE)
      return FALSE;
    $netmask = decbin ($netmask);
    for ($i=0 ; $i<32 ; $i++)
    {
      if ($netmask{$i} == 0)
      {
        // Fin du masque : on sort
        break;
      }
    }
    return $i;
  }

  function broadcast ($ip, $cidr)
  {
    // Cette fonction renvoie l'adresse IP de broadcast en fonction de l'ip et
    // du CIDR passe.
    // Ex. Renvoie 192.168.0.255 si $ip=192.168.0.0 et $cidr=24
    $ip_cidr = ip2long ($cidr);
    $ip_ip = ip2long ($ip);
    $ip_mask = (1 << (32 - $cidr)) - 1;
    return long2ip($ip_ip | $ip_mask);
  }

  function interfaceRoutage ($adresseIP)
  {
    // Cette fonction regarde dans la table de routage de la machine par quelle
    // interface le paquet a destination de $adresse ira.
    // $adresse est donc une IP en /32.
    // Renvoie l'interface physique et le CIDR associe
    // Renvoie FALSE si la table de routage est illisible dans /proc/net/route
    // Renvoie FALSE si $adresseIP est incorrecte
    if (!function_exists("criclog"))
      die ("La librairie logger n'est pas chargée dans net\n");
    $file = file ("/proc/net/route",
                                   FILE_SKIP_EMPTY_LINES|FILE_IGNORE_NEW_LINES);
    if ($file === FALSE)
    {
      criclog ("ERREUR interfaceRoutage ($adresseIP) : /proc/net/route ".
               "illisible");
      return FALSE;
    }
    // Il n'y a pas la loopback dans le routage, on la rajoute
    $file[] = "lo 0000007F 0100007F 001 0 0 0 000000FF 0 0 0";
    $adresse = sprintf("%u", ip2long ($adresseIP));
    if ($adresse === FALSE)
    {
      criclog ("ERREUR interfaceRoutage ($adresseIP) : \$adresse === FALSE");
      return FALSE;
    }
    $MaxMasque = sprintf ("%u", ip2long ("0.0.0.0"));
    $interface = FALSE;
    foreach ($file as $ligne)
    {
      $ligne = rtrim ($ligne);
      list ($iface, $dest, $gateway, $flags, $refCnt, $use, $metric, $masque,
            $reste) = sscanf($ligne, "%s %s %s %s %s %s %s %s %s");
      if ($iface === "Iface")
      {
        // On saute le titre
        continue;
      }
      // Les nombres ne sont pas dans le bon ordre : on re-aligne dans l'ordre
      // normal 0100007F -> 7F000001 (1.0.0.127 -> 127.0.0.1). Ce changement est
      // important pour la priorite des masques (les bits de poids fort doivent
      // etre en tete)
      $dest = hexdec ($dest{6}.$dest{7}.$dest{4}.$dest{5}.$dest{2}.$dest{3}.
                      $dest{0}.$dest{1});
      $masque = hexdec ($masque{6}.$masque{7}.$masque{4}.$masque{5}.$masque{2}.
                        $masque{3}.$masque{0}.$masque{1});
      $reseau = $dest & $masque;
      $adresseMasquee = sprintf ("%u",((float)$adresse & (float)$masque));
      $destMasquee = sprintf ("%u",((float)$dest & (float)$masque));

      if ($adresseMasquee === $destMasquee)
      {
        // l'adresse rentre dans ce masque. On verifie si ce nouveau masque est
        // plus serre que l'ancien vu
        if ($masque >= $MaxMasque)
        {
          $interface = $iface;
          $MaxMasque = $masque;
        }
      }
    }
    return array($interface, netmask2cidr($MaxMasque));
  }

  function arpExiste ($arpOuIp)
  {
    // Cette fonction cherche dans la table arp si l'adresse $arp est bien
    // disponible. Renvoie FALSE si l'adresse  MAC est introuvable, l'IP
    // associee si l'adresse MAC y est. Si on a passe une IP, renvoie l'adresse
    // MAC associee.
    $arpOuIp = strtoupper ($arpOuIp);
    if (!file_exists ("/proc/net/arp"))
      return FALSE;
    $listeMacFile = file ("/proc/net/arp",   FILE_IGNORE_NEW_LINES |
                                             FILE_SKIP_EMPTY_LINES);
    foreach ($listeMacFile as $ligne)
    {
      if ($ligne === "IP address       HW type     Flags       HW address   ".
                     "         Mask     Device")
      {
        // on saute la ligne de titre
        continue;
      }
      list ($ip, $type, $flags, $hwaddr, $mask, $device) =
                                           sscanf ($ligne, "%s %s %s %s %s %s");
      if ($flags === "0x0")
      {
        // Adresse incomplete
        continue;
      }
      if ($hwaddr === $arpOuIp)
        return $ip;
      if ($ip === $arpOuIp)
        return $hwaddr;
    }
    return FALSE;
  }

  ///////////
  // TESTS //
  ///////////
/*
  var_dump(__listeInterface());
  var_dump (ipActiveeInterface ("eth4", "10.10.10.22"));
  var_dump (ipActiveeInterface ("eth0", "10.10.10.10"));
  var_dump (ipActiveeInterface ("lo:0", "10.10.10.12"));
  var_dump (listeIP());
  var_dump (cidr2netmask (24));
  var_dump (cidr2netmask (27));
  var_dump (cidr2netmask (32));
  var_dump (broadcast("192.168.0.12", 31));
  var_dump (interfaceReelle ("eth0"));
  var_dump (interfaceReelle ("eth0:9"));
  var_dump (interfaceReelle ("lo:9"));
  var_dump ( netmask2cidr ("255.255.255.255"));
  var_dump ( netmask2cidr ("255.255.255.0"));
  var_dump ( netmask2cidr ("0.0.0.0"));

  echo "127.1.1.1\n";
  var_dump (interfaceRoutage ("127.1.1.1"));
  echo "10.10.10.10\n";
  var_dump (interfaceRoutage ("10.10.10.10"));
  echo "10.10.10.15\n";
  var_dump (interfaceRoutage ("10.10.10.15"));
  echo "1.2.3.4\n";
  var_dump (interfaceRoutage ("1.2.3.4"));
  echo "10.0.0.25\n";
  var_dump (interfaceRoutage ("10.0.0.25"));
  echo "10.10.3.15\n";
  var_dump (interfaceRoutage ("10.10.3.15"));
  echo "255.255.255.255\n";
  var_dump (interfaceRoutage ("255.255.255.255"));

  echo ipOuNom("1.2.3.4")."\n";
  echo ipOuNom("1.2.3.4:1111")."\n";
  echo ipOuNom("mac.domain.tld")."\n";
  echo ipOuNom("mac.domain.tld:1111")."\n";

  $ip="147.173.1.226";
  $ping=ping($ip,$erreur, $timeout);
  if ($ping!==FALSE)
  {
    // OK Le test a répondu dans le temps imparti
    echo "Réponse : ".$ping."s\n";
  }
  else
  {
    // ERREUR Le test n'a pas répondu dans le temps imparti ou une erreur
    // est générée
    echo $erreur."\n";
  }
  $rc=commandeDistante ("localhost", "8000", "echo 'coucou\n';");
  if ($rc===FALSE)
    echo "FALSE\n";
  else
    echo "--$rc--\n";
  $rc=commandeDistante ("localhost", "8000", "echo 'coucou';");
  if ($rc===FALSE)
    echo "FALSE\n";
  else
    echo "--$rc--\n";
  smtp_mail("user@adresse.fr,user2@adresse2.com", "test", "test");
  smtp_mail("dominique.fournier@grenoble.cnrs.fr", "test", "test");

  var_dump(pingUp ("147.173.1.254", 3, 3));
  var_dump(pingUp ("147.173.1.254", 1, 1));
  var_dump(pingUp ("147.173.1.54", 1, 1));
  var_dump(pingUp ("147.173.1.54", 1, 3));

  var_dump (testTCP("smtp-gw2.grenoble.cnrs.fr", 25, "QUIT", 0, 1));
  var_dump (testTCP("sec2", 53, "", 0, 0));
  var_dump (testTCP("localhost", 80, "GET /", 0, 0));
  var_dump (testTCP("localhost", 443, "GET /", 1, 0));
  var_dump (testTCP("localhost", 3306, "\n\n", 0, 0));

  var_dump (arpExiste ("sdgsdgsd"));          // FALSE
  var_dump (arpExiste ("BE:40:E4:9D:6B:80")); // 10.10.10.254
  var_dump (arpExiste ("be:40:e4:9d:6b:80")); // 10.10.10.254
  var_dump (arpExiste ("10.10.10.254"));      // BE:40:E4:9D:6B:80

$ip="147.173.1.15";
// Test DNS firewall : Repond -1 au bout de 3 secondes
var_dump (gethostbyaddr_timeout($ip, "147.173.1.15", 3));
// Test DNS  OK : renvoie 0 immediatement
var_dump (gethostbyaddr_timeout($ip, "147.173.1.26", 3));
// Test DNS adresse inexistante : renvoie -2
var_dump (gethostbyaddr_timeout("1.2.3.4", "147.173.1.26", 3));

  var_dump (testNFSv4TCP ("10.10.10.101", 2049, 3));
  var_dump (testPortmapTCP ("10.10.10.101", 111, 3));
*/
?>
