<?
  // smtp.php

  // Envoi des mails

  error_reporting(E_ALL | E_STRICT);

  /////////////////////////
  // FONCTIONS PUBLIQUES //
  /////////////////////////
  function smtp_mail ($to, $from, $subject, $message_contenu, $smtp_host, 
                      $attachs = array(), $headers = "", $smtp_user="",
                      $smtp_pass="")
  {
    // Envoie le mail aux personnes du champ $to, depuis l'adresse $from, de
    // sujet $subject. Le message peut etre en texte ou en HTML. Les fichiers
    // joints doivent etre de la forme
    // $attach[] = array ('nom'=>'nom du fichier', 'contenu'=>'contenu', 
    //                    'attach'=>'inline',
    //                    'mime-type'=>'image/png; charset=binary');
    // Si contenu n'est pas defini, on ira chercher le fichier sur le disque.
    // Le nom peut etre un chemin absolu vers un fichier.
    // Le champ 'attach' est optionnel : par defaut, on met en attache.
    // Par contre, si il vaut 'inline' et que le message est du HTML, on place
    // ce fichier comme fichier inline HTML.
    // Si le message est du texte, le fichier de type 'inline' est ignore.
    // La page HTML doit comporter les images passees en fichiers inline. Les
    // images doivent etre codees en : 
    //  <img src='cid:NomDuFichierImage.ext'/> (on peut remplacer ' -> ")
    // Le NomDuFichierImage.ext est le basename de l'image passe dans la liste
    // des fichiers a mettre inline
    // Le 'mime-type' est optionnel, si le PHP possede le support de fileinfo.
    // Si ce n'est pas le cas, il est obligatoire.

    // Retourne TRUE si le mail a bien ete envoye
    // Retourne FALSE en cas d'echec
    $preferences = array(
      "input-charset" => "UTF-8",
      "output-charset" => "UTF-8",
      "line-length" => 75, // 75 pour toujours avoir une place pour une virgule
      "line-break-chars" => "\r\n",
      "scheme" => "Q",
    );
    $message = "";

    //--- Gestion de l'expediteur ---//
    if (($pos = strpos ($from, "<")) !== FALSE)
    {
      // Le mail est de type 'Dominique Fournier <dominique@fournier38.fr>
      // Le SMTP FROM ne veut que dominique@fournier38.fr
      $email_from = substr (trim ($from), $pos+1, -1);
      $fullname_from = trim (substr (trim ($from), 0, $pos));
    }
    else
    {
      // Le mail est de type 'dominique@fournier38.fr'
      $email_from = $from;
      $fullname_from = "";
    }
    // Conversion du Full Name (accentue) en Format Mime : (meme si il est vide
    // pour avoir le nom du champ en tete), mais sauf si il est en ASCII
    if (__smtp_is_ascii ($fullname_from))
      $fullname_from = "From: $fullname_from";
    else
      $fullname_from = iconv_mime_encode("From", $fullname_from, $preferences);
    $message .= wordwrap ("$fullname_from <$email_from>", 76, "\r\n ")."\r\n";

    //--- Gestion des destinataires ---//
    $recipients = explode(",", $to);
    $email_tos = array();
    $i = 1;
    foreach ($recipients as $email)
    {
      // Email peut etre de la forme : 
      // - Dominique Fournier <dominique@fournier38.fr>
      // - dominique@fournier38.fr
      $email = trim ($email);
      if (($pos = strpos ($email, "<")) !== FALSE)
      {
        // Le mail est de type 'Dominique Fournier <dominique@fournier38.fr>
        // Le RCPT TO ne veut que dominique@fournier38.fr
        // On doit encoder le fullname si il comporte des caracteres non ascii
        $email_to = substr ($email, $pos+1, -1);
        $fullname_to = trim (substr (trim ($email), 0, $pos));
        // La premiere entree compore le nom du champ 'To: ', mais pas les
        // suivantes. On convertit chaque nom complet en mode Q, sauf si c'est
        // de l'ASCII pur
        if (__smtp_is_ascii ($fullname_to))
        {
          if ($i === 1)
            $fullname_to = "To: $fullname_to";
          else
            $fullname_to = " $fullname_to";
        }
        else
        {
          if ($i === 1)
            $fullname_to = iconv_mime_encode("To", $fullname_to, $preferences);
          else
            $fullname_to = substr (iconv_mime_encode("", $fullname_to,
                                                         $preferences), 1);
        }
        $message .= wordwrap ("$fullname_to <$email_to>", 76, "\r\n ");
      }
      else
      {
        // Le mail est de type 'dominique@fournier38.fr'
        $email_to = $email;
        $fullname_to = "";
        if ($i === 1)
          $message .= "To: ". wordwrap ("$email_to", 72, "\r\n ");
        else
          $message .= wordwrap (" $email_to", 76, "\r\n ");
      }
      if ($i < count ($recipients)) $message .= ",";
      $message .= "\r\n";

      $email_tos[] = $email_to;
      $i++;
    }

    //--- Suject doit etre code, sauf si c'est de l'ASCII ---//
    if (__smtp_is_ascii ($subject))
      $message .= wordwrap ("Subject: $subject", 76, "\r\n ")."\r\n";
    else
      $message .= iconv_mime_encode ("Subject", $subject, $preferences)."\r\n";

    //--- Ajoute la date aux entetes ---//
    $message .= "Date: ".date ("r")."\r\n";

    // On regarde les fichiers qui sont en mode 'inline' pour les mettre avec
    // le HTML
    $attachInline = array();
    foreach ($attachs as $key => $attach)
    {
      if (isset ($attach['attach']) && $attach['attach'] === 'inline')
      {
        $attachInline[] = $attach;
        unset ($attachs[$key]);
      }
    }

    //--- Gestion de la fin de l'entete en fonction du contenu ---//
    $boundary = uniqid ('', true);
    if (empty ($attachs))
    {
      // Pas de piece jointe (ou alors en inline), on place le message 
      // directement (en alternative si c'est du HTML)
      if (stripos ($message_contenu, '<html') !== FALSE)
      {
        // Le message est du HTML
        $message .= __smtp_messageHTML ($message_contenu, $attachInline);
      }
      else
      {
        // Le message est du texte pur
        $message .= "Content-Type: text/plain; charset=UTF-8\r\n".
                    "Content-Transfer-Encoding: 8bit\r\n\r\n";
        $message .= $message_contenu."\r\n\r\n";
      }
    }
    else
    {
      // Des pieces jointes : on place l'entete en mixed, et le message sera
      // dans la premiere boundary (et en alternative si c'est du HTML)
      $message .= "MIME-Version: 1.0\r\n";
      $message .= "Content-Type: multipart/mixed;\r\n".
                  " boundary=$boundary\r\n\r\n";
      $message .= "This is a multi-part message in MIME format.\r\n";
      $message .= "--$boundary\r\n";

      if (stripos ($message_contenu, '<html') !== FALSE)
      {
        // Le message est du HTML
        $message .= __smtp_messageHTML ($message_contenu, $attachInline)."\r\n";
      }
      else
      {
        // Le message est du texte
        $message .= "Content-Type: text/plain; charset=UTF-8\r\n".
                    "Content-Transfer-Encoding: 8bit\r\n\r\n";
        $message .= $message_contenu."\r\n\r\n";
      }

      $message .= __smtp_Fichiers_Joints ($attachs, $boundary);
      $message .= "--$boundary--\r\n";
    }
    // Le message contient la fin de l'entete, separe du corps avec tout le 
    // contenu par une ligne blanche.
    // $email_tos est la liste des emails seuls dans un tableau
    // $email_from est l'adresse email de l'emetteur seule
   
    return __smtp_mail_generique ($email_tos, $email_from, $message, 
                                  $smtp_host,
                                  $headers, $smtp_user, $smtp_pass);
  }

  ///////////////////////
  // FONCTIONS PRIVEES //
  ///////////////////////
  function __smtp_Fichiers_Joints ($attachs, $boundary, $inline=FALSE)
  {
    // Cette fonction se charge de placer tous les fichiers joints avec encodage
    // si necessaire. La boundary est placee entre chaque fichier (si il y en a
    // plusieurs)
    $message = "";
    foreach ($attachs as $key => $attach)
    {
      // Pour chaque fichier joint, il faut mettre l'entete. Si le fichier est
      // du texte pur, on ne le code pas. Sinon, on le transforme en base64.
      // Les noms doivent etre codes en format mime si ils comportent des
      // accents
      if (!isset ($attach['nom']))
        continue;
      $message .= "--$boundary\r\n";
      if (!isset ($attach['contenu']))
      {
        if (is_readable ($attach['nom']))
        {
          $contenu = file_get_contents ($attach['nom']);
          $attach['nom'] = basename ($attach['nom']);
        }
        else
        {
          trigger_error ("__smtp_Fichiers_Joints : ".$attach['nom'].
                         " not found", E_USER_ERROR);
        }
      }
      else
      {
        $contenu = $attach['contenu'];
      }
      if (substr ($contenu, -1) !== "\n")
        $contenu .= "\r\n";
      if (function_exists ("finfo"))
      {
        $finfo = new finfo (FILEINFO_MIME);
        $typeMime = $finfo->buffer ($contenu);
      }
      elseif (isset ($attach['mime-type']))
      {
        $typeMime = $attach['mime-type'];
      }
      else
      {
        echo "ERREUR : Support Fileinfo non dispo et pas de 'mime-type' passé ".
             "pour ".$attach['nom']."\n";
      }

      if (__smtp_is_ascii ($attach['nom']))
        $nom = $attach['nom'];
      else
        $nom = substr (iconv_mime_encode ("", $attach['nom'], $preferences),
                       1);
      $message .= "Content-Type: $typeMime;\r\n";
      $message .= " name=\"$nom\"\r\n";
      if (substr ($typeMime, 0, 4) === "text")
      {
        $message .= "Content-Transfer-Encoding: 8bit\r\n";
      }
      else
      {
        $message .= "Content-Transfer-Encoding: base64\r\n";
        $contenu = chunk_split (base64_encode ($contenu));
      }
      if ($inline === TRUE)
      {
        $message .= "Content-ID: <$nom>\r\n";
        $message .= "Content-Disposition: inline;\r\n";
      }
      else
        $message .= "Content-Disposition: attachment;\r\n";
      
      $message .= " filename=\"$nom\"\r\n";

      $message .= "\r\n";
      $message .= $contenu."\r\n";
    }
    return $message;
  }

  function __smtp_messageHTML ($messageHTML, $attachsInline=array())
  {
    // Cette fonction convertit le message HTML en texte et genere 
    // l'architecture multipart/alternative necessaire a l'insertion dans le 
    // mail
    // Renvoie le message avec les boundaries necessaires

    $boundary = uniqid ('', true);

    $patterns = array();
    $patterns[0] = '@<i>@i';
    $patterns[1] = '@</i>@i';
    $patterns[2] = '@<b>@i';
    $patterns[3] = '@</b>@i';
    $patterns[4] = '@<u>@i';
    $patterns[5] = '@</u>@i';
    $patterns[6] = '@<head>.*</head>@ism'; // Enleve tout l'entete pour ne pas voir le
                                           // titre ou la feuille de style dans les 
                                           // textes
    $remplacements = array ();
    $remplacements[0] = '/';
    $remplacements[1] = '/';
    $remplacements[2] = '*';
    $remplacements[3] = '*';
    $remplacements[4] = '_';
    $remplacements[5] = '_';
    $remplacements[6] = '';
    $messageTXT = trim (strip_tags (
                    preg_replace ($patterns, $remplacements, $messageHTML)
                  ));
    $message  = "";
    $message .= "Content-Type: multipart/alternative;\r\n".
                " boundary=$boundary\r\n\r\n";
    $message .= "--$boundary\r\n";
    $message .= "Content-Type: text/plain; charset=UTF-8\r\n".
                "Content-Transfer-Encoding: 8bit\r\n\r\n";
    $message .= $messageTXT."\r\n\r\n";
    $message .= "--$boundary\r\n";
    if (empty ($attachsInline))
    {
      $message .= "Content-Type: text/html; charset=UTF-8\r\n".
                  "Content-Transfer-Encoding: 8bit\r\n\r\n";
      $message .= $messageHTML."\r\n\r\n";
    }
    else
    {
      $boundary2 = uniqid ('', true);
      $message .= "Content-Type: multipart/related;\r\n".
                  " boundary=$boundary2\r\n\r\n";
      $message .= "--$boundary2\r\n";
      $message .= "Content-Type: text/html; charset=UTF-8\r\n".
                  "Content-Transfer-Encoding: 8bit\r\n\r\n";
      $message .= $messageHTML."\r\n\r\n";
      $message .= __smtp_Fichiers_Joints ($attachsInline, $boundary2, TRUE);
      $message .= "--$boundary2--\r\n\r\n";
    }
    $message .= "--$boundary--\r\n";
    return $message;
  }

  function __smtp_is_ascii ($txt)
  {
    // Cette fonction verifie si le texte passe en argument ne compore que des
    // caracteres ASCII
    // Renvoie TRUE si c'est de l'ASCII
    // Renvoie FALSE si c'est de l'UTF-8 ou de l'ISO-88...
    // Une chaine vide est en ASCII.
    for ($i = 0 ; $i < strlen ($txt) ; $i++)
    {
      if (ord ($txt{$i}) < 0 ||ord ($txt{$i}) > 127)
        return FALSE;
    }
    return TRUE;
  }

  function __smtp_mail_generique ($email_tos, $email_from, $message,
                                  $smtp_host, $headers = "\r\n", $smtp_user="", 
                                  $smtp_pass="")
  {
    // Les $to doivent etre les mails SEULS dans un tableau. Il FAUT qu'ils
    // concernent tous le domaine auquel on se connecte.
    // Cette fonction attend dans $message les entetes necessaires et le corps
    // du mail

    // Gestion des ports TCP speciaux ou 25 par defaut
    if (substr ($smtp_host, 0, 6) === "ssl://")
    {
      // Connexions SMTPS
      $host = substr ($smtp_host, 6);
      if (strpos ($host, ":") !== FALSE)
      {
        list($smtp_host, $smtp_port) = explode(":", $host);
        $smtp_host = "ssl://$smtp_host";
      }
      else
      {
        $smtp_port = 465;
      }
    }
    else
    {
      // Connexions SMTP
      if (strpos ($smtp_host, ":") !== FALSE)
      {
        list ($smtp_host, $smtp_port) = explode(":", $smtp_host);
      }
      else
      {
        $smtp_host = $smtp_host;
        $smtp_port = 25;
      }
    }

    // Calcul du nom de cette machine (pour le EHLO)
    if (is_readable ("/etc/hostname"))
      $smtp_ehlo = trim (file_get_contents ("/etc/hostname"));
    else
      $smtp_ehlo = "localhost.localdomain";

    // Et d'un message ID
    $headers = "Message-Id: <".uniqid ('', true)."@$smtp_ehlo>\r\n$headers";

    $content = "$headers$message\r\n";

    // Le mail est pret : on l'envoie
    if (!($socket = @fsockopen($smtp_host, $smtp_port, $errno, $errstr, 5)))
    {
      if (function_exists("criclog"))
        criclog ("Could not connect to smtp host '$smtp_host' (".
                $errno.") (".$errstr.")", __FILE__, __LINE__);
      else
        trigger_error ("Could not connect to smtp host '$smtp_host' (".
                $errno.") (".$errstr.")". __FILE__. __LINE__, E_USER_ERROR);
      return FALSE;
    }
    stream_set_timeout  ($socket, 5);

    if (!__smtp_server_parse ($socket, "220")) return;

    fwrite ($socket, "EHLO ".$smtp_ehlo."\r\n");
    if (!__smtp_server_parse ($socket, "250")) return;

    if ($smtp_user !== "" && $smtp_pass !== "")
    {
      fwrite ($socket, "AUTH LOGIN"."\r\n");
      if (!__smtp_server_parse ($socket, "334")) return;

      fwrite ($socket, base64_encode ($smtp_user)."\r\n");
      if (!__smtp_server_parse ($socket, "334")) return;

      fwrite ($socket, base64_encode ($smtp_pass)."\r\n");
      if (!__smtp_server_parse($socket, "235")) return;
    }

    fwrite ($socket, "MAIL FROM: <$email_from>\r\n");
    if (!__smtp_server_parse ($socket, "250")) return;

    foreach ($email_tos as $email_to)
    {
      fwrite ($socket, "RCPT TO: <".$email_to.">"."\r\n");
      if (!__smtp_server_parse ($socket, "250")) return;
    }

    fwrite ($socket, "DATA"."\r\n");
    if (!__smtp_server_parse ($socket, "354")) return;


    fwrite ($socket, $content);

    fwrite ($socket, ".\r\n");
    if (!__smtp_server_parse ($socket, "250")) return;

    fwrite ($socket, "QUIT"."\r\n");
    fclose ($socket);

    return TRUE;
  }

  function __smtp_server_parse ($socket, $expected_response)
  {
    $server_response = "";
    $startTime = microtime(TRUE);
    while ($startTime+2.00 > microtime(TRUE) &&
           substr($server_response, 3, 1) !== " ")
    {
      $server_response = fgets($socket, 256);
    }
    if ($server_response === FALSE)
    {
      if (function_exists("criclog"))
        criclog ("Serveur SMTP HS pendant la communication : dehors");
      else
        trigger_error ("Serveur SMTP HS pendant la communication : dehors",
                       E_USER_ERROR);
      return FALSE;
    }

    if (! (substr ($server_response, 0, 3) === $expected_response))
    {
      if (function_exists("criclog"))
        criclog ("Erreur d'envoi de mail par SMTP : $server_response");
      else
        trigger_error ("Erreur d'envoi de mail par SMTP : $server_response",
                       E_USER_ERROR);
 
      return FALSE;
    }
    return TRUE;
  }

  /////////////////////
  // TESTS UNITAIRES //
  /////////////////////
  /*$conf['loggerFile'] = "/tmp/log";
  $conf['timezone'] = 'Europe/Paris';
  require ("../ErreursLogs/logger.php");
  var_dump (smtp_mail (
// To
" Domrìnique Fournier <dominique@fournier38.fr>, dominique@fournier.homedns.org",
// From
"Dominique Fourníer <dominique.fournier@grenoble.cnrs.fr>",
// Sujet
"Bla bla de plus de 76 caractêres mais purement ascii, non mais, je reve de finir",
//"Test accentué plus long que l'énorme mail qui est juste au dessus !!! êeee",
// Contenu
"Bla Bla accentué aussi",
"192.168.1.253"));
  var_dump (smtp_mail ("Dominique Fournier <dominique.fournier@grenoble.cnrs.fr>,".
" Dominique Fourniér <dominique@fournier38.fr>, dominique@fournier.homedns.org",
"Dominique Fourniér <dominique.fournier@grenoble.cnrs.fr>",
"Test accentué",
"Bla Bla accentué aussi",
"192.168.1.253"));*/
?>
