<?
  // processus.php

  // Fonctions necessaires a la gestion des processus.

  error_reporting(E_ALL | E_STRICT);

  ////////////////////////////////
  // VERIFICATIONS DE DEMARRAGE //
  ////////////////////////////////
  if (!function_exists("criclog"))
    die ("La librairie logger n'est pas chargee dans processus.php\n");

  ///////////////
  // FONCTIONS //
  ///////////////
  function startBackground($prog, $args=array())
  {
    // Demarre le programme $prog en tant que fils de ce programme. Si des
    // arguments sont necessaires, ils sont places dans l'array $args
    // Renvoie le PID du fils
    $fh_unused = array(STDIN, STDOUT);
    $pid = pcntl_fork();
    if ($pid == -1)
    {
         die('could not fork');
    }
    elseif ($pid)
    {
      // we are the parent : on retourne le PID du fils
      return ($pid);
    } else {
      // we are the child : on lance le programme externe
      $pid = posix_getpid();
      // Devient session leader (voir setsid(2))
      $sid = posix_setsid();
      if ($sid==-1)
      {
        criclog ("ERREUR : $pid n'a pas pu se détacher du terminal");
        criclog ("FILS SID($pid)=".posix_getsid($pid));
      }

      // Clot les file handlers STDOUT/STDIN
      @fclose (STDIN);
      @fclose (STDOUT);
      @fclose (STDERR);

      $rc=pcntl_exec($prog,$args);
      if ($rc===FALSE)
      {
        criclog ("ERREUR DE LANCEMENT DE $prog : ${php_errormsg}");
      }
      exit;
    }
  }

  function execlocal ($programme, $warning=0)
  {
    // Cette fonction execute le programme passe et renvoie dans le log si $rc
    // est different de 0
    $programme.=" 2>&1";
    trigger_error ("$programme", E_USER_WARNING);
    unset ($output);
    exec ($programme, $output, $rc);
    if ($rc!=0)
    {
      if ($warning==0)
        criclog ("ERREUR : '$programme' a renvoyé le rc=$rc et le stdout :");
      else
        criclog ("ATTENTION : '$programme' a renvoyé le rc=$rc et le stdout :");
      criclog ($output);
      sleep (3);
    }
    unset ($output);
    return $rc;
  }

  // Variable Globale pour sauter les fonctions sig_handler_local et
  // execAvecWarning
  $rc_execAvecWarning=0;
  function sig_handler_local()
  {
    global $rc_execAvecWarning;
    // On n'appelle le sig_handler que lorsque le fils est mort
    pcntl_wait($status);
    $rc_execAvecWarning=pcntl_wexitstatus($status);
  }

  function execAvecWarning ($programme, $args=array(), $heartbeat="")
  {
    //Execute le programme $programme et met un log toutes les 10s
    declare(ticks = 1);
    global $rc_execAvecWarning, $argv;
    pcntl_signal(SIGCHLD, "sig_handler_local");
    $pid=startBackground($programme, $args);
    $i=0;
    while (file_exists("/proc/$pid/"))
    {
      if ($i%10==9)
      {
        criclog("En attente de fin du processus '$programme'");
        // On touche le fichier de heartbeat pour eviter le timeout (le
        // programme principal n'est pas plante, il est en attente d'un autre
        // programme)
        if ($heartbeat !== "") 
          @touch ($heartbeat);
        $i=0;
      }
      else
      {
        $i++;
      }
      sleep (1);
    }
    // Le processus fils est mort et donc le signal CHLD a ete appelle et la
    // variable globale $rc_execAvecWarning a ete definie : on sort
    return $rc_execAvecWarning;
  }

  function processusActif ($processus)
  {
    // Recherche dans /proc si le processus dont le nom est passe en argument
    // est bien actif
    // lecture de /proc pour voir si ldirectord est lancé
    if (is_dir("/proc"))
    {
      if ($proc = opendir("/proc"))
      {
        $stop = false;
        while (($f = readdir($proc)) && !$stop)
        {
          // ouverture d'un fichier particulier dans chaque repertoire
          $chemin = "/proc/$f/cmdline";
          if ((file_exists($chemin)) && ($buff=file_get_contents($chemin)))
          {
            // analyse de la commande pour voir si c'est ldirectord
            if (strpos($buff,$processus)!==FALSE)
            {
              $stop = true;
            }
          }
        }
        return $stop;
      }
    }
    return false;
  }

  ///////////
  // TESTS //
  ///////////
/*
  var_dump (processusActif("php"));
  var_dump (processusActif("ldirectord"));
*/
?>
