<?php

  // rateLimit.php

  function rateLimitSHM ($data, $nbSeen, $time, $identifier)
  {
    // Return TRUE if the $data string is not seen $nbSeen in $time seconds
    // Return FALSE elsewhere
    // $data can be an client IP address to be limited...
    // Use semaphores and shared memory, so available only on Unixes
    // Can be used by multiple threads so locking is possible
    // One record by microsecond. With the lock, the second processus of the
    // same microsecond will wait 1us before recording
    $rc = TRUE;
    $time = $time * 1000000; // Put it in microsecond
    touch ("/tmp/rateLimit-$identifier");
    $SEMKey = ftok ("/tmp/rateLimit-$identifier", "F");
    $SHMKey = ftok ("/tmp/rateLimit-$identifier", "M");
    $seg = sem_get ($SEMKey, 1, 0666, -1);
    $shm_id = shm_attach ($SHMKey);
    // Block here until the exclusive lock is given to me
    sem_acquire ($seg);
    // Shared Memory is locked : get values
    $all = unserialize (@shm_get_var ($shm_id, 0));
    if ($all === FALSE)
    {
      //echo "Init shared memory table\n";
      $all = array();
    }
    // Removing old values
    foreach ($all as $timestamp=>$ip)
    {
      if ($timestamp < microtime(true)*1000000 - $time)
        unset ($all[$timestamp]);
    }
    
    if (isset ($all[microtime(true)*1000000]))
    {
      // Timestamp collision, wait 2us
      usleep (2);
    }

    // Check if the $data is seen 
    $countArray = array_count_values ($all);
    if (isset ($countArray[$data]) && $countArray[$data] >= $nbSeen)
    {
      //echo "Rate limit wait at least $time microseconds\n";
      $rc = FALSE;
      // To block until a stop is done, remove the older information
      unset ($all[array_search ($data, $all)]);
    }
    // Save the new last connection
    $all[microtime(true)*1000000] = $data;

    // Save new values into shared memory
    shm_put_var ($shm_id, 0, serialize ($all));

    shm_detach ($shm_id);
    sem_release ($seg);
    return $rc;
  }

  function readRateLimitSHMTable ($identifier)
  {
    // This function display the content of rateLimit shared memory
    touch ("/tmp/rateLimit-$identifier");
    $SHMKey = ftok ("/tmp/rateLimit-$identifier", "M");
    $shm_id = shm_attach ($SHMKey);
    $all = unserialize (shm_get_var ($shm_id, 0));
    print_r ($all);
    $countArray = array_count_values ($all);
    print_r ($countArray);
  }

  function readRateLimitMemcached ($data, $time, $identifier, $memcachedServer,
                                   $memcachedPort=11211)
  {
    // Display the content of Memcached on the $time period since now and for
    // $data 
    $keys = array();
    for ($i = time() - $time * 60 ; $i <= time() ; $i += 60)
    {
      $keys[] = "$identifier$data-".date ("Y/m/d-H:i", $i);
    }
    if (class_exists ('Memcached'))
    {
      $m = new Memcached();
      $m->addServer($memcachedServer, $memcachedPort, TRUE);
      $vals = $m->getMulti ($keys);
    }
    else
    {
      $m = new Memcache();
      // On se connecte au serveur en mode persistent.
      // On vire l'erreur pour eviter les messages intempestifs si le serveur
      // a un probleme et qu'on ne peut pas se connecter
      @$m->pconnect($memcachedServer, $memcachedPort);
      $vals = $m->get ($keys);
    }
    unset ($m);
    print_r ($vals);
  }

  function rateLimitMemcached ($data, $nbSeen, $time, $identifier, 
                               $memcachedServer, $memcachedPort=11211) 
  {
    // Return TRUE if the $data string is not seen $nbSeen in $time minutes
    // Return FALSE if the $data isn't overflow ratelimiter.

    // Keep for each last $time minute the number of connection.
    // Ask to memcached the number of connection for these $time minutes, add
    // the number and compare with $nbSeen.

    // In case of problem with the rate-limit server, let the connection go, 
    // return TRUE. There is no ratelimit until the server will be relaunched

    // $time is in minutes, not in seconds, to limit the number of requests, and
    // the number of records in the Memcached database. Instead, use the
    // incremental method

    // Memcache don't like spaces in $data : change them to underscore
    $data = str_replace (" ", "_", $data);

    if (!class_exists ('Memcached') && !class_exists ('Memcache'))
    {
      // Block all the connections to be sure that the adminitrator move
      return FALSE;
    }

    // Initialization of the connection to Memcached server
    $key = "$identifier$data-".date ("Y/m/d-H:i");
    if (class_exists ('Memcached'))
    {
      $m = new Memcached();
      $m->addServer($memcachedServer, $memcachedPort, TRUE);
      // Add create the value but if it doesn't already exists
      $m->add ($key, 0, time() + $time*60);
    }
    else
    {
      $m = new Memcache();
      // On se connecte au serveur en mode persistent.
      // On vire l'erreur pour eviter les messages intempestifs si le serveur
      // a un probleme et qu'on ne peut pas se connecter
      @$m->pconnect($memcachedServer, $memcachedPort);
      // Add create the value but if it doesn't already exists
      $m->add ($key, 0, NULL, time() + $time*60);
    }

    $incr = $m->increment ($key);
    if ($incr === FALSE)
      return TRUE;

    // Check the number of connections on the last $time minutes
    $keys = array();
    for ($i = time() - $time * 60 ; $i <= time() ; $i += 60)
    {
      $keys[] = "$identifier$data-".date ("Y/m/d-H:i", $i);
    }
    if (class_exists ('Memcached'))
      $vals = $m->getMulti ($keys);
    else
      $vals = $m->get ($keys);
    $cmpt = array_sum ($vals);

    unset ($m);

    if ($cmpt > $nbSeen)
      return FALSE;
    return TRUE;
  }
?>
