<?php
  // strings.php

  function is_ascii ($txt)
  {
    // Cette fonction verifie si le texte passe en argument ne compore que des
    // caracteres ASCII
    // Renvoie TRUE si c'est de l'ASCII
    // Renvoie FALSE si c'est de l'UTF-8 ou de l'ISO-88...
    // Une chaine vide est en ASCII.
    for ($i = 0 ; $i < strlen ($txt) ; $i++)
    {
      if (ord ($txt{$i}) < 0 ||ord ($txt{$i}) > 127)
        return FALSE;
    }
    return TRUE;
  }

  function stringQuotedToUTF ($text)
  {
    // Cette fonction prend une chaine quotee avec les codages et la convertit 
    // en UTF-8
    $debug=0;
    if ($debug) echo "stringQuotedToUTF ($text)\n";
    $elements = imap_mime_header_decode($text);
    if ($debug) print_r ($elements);
    $chaineRet = "";
    for ($i=0; $i<count($elements); $i++)
    {
      if ($elements[$i]->charset=="default")
        $elements[$i]->charset="iso-8859-1";
      $chaineRet .= iconv ($elements[$i]->charset, "UTF-8",
                           $elements[$i]->text);
    }
    return $chaineRet;
  }

  function ucfirstmultiple ($prenom)
  {
    // Cette fonction renovie le premier caractere de chaque mot en majuscule.
    // Les mots sont detectes par les espaces, tirets, points et apostrophes
    // Exemple d'utilisation : Harmonisation des prenoms
    $prenom=strtolower($prenom);
    $prenoms=explode(" ",$prenom);
    $prenom=mb_convert_case($prenom, MB_CASE_TITLE, "UTF-8");
    $prenoms=explode("-",$prenom);
    foreach ($prenoms as $value)
      $pren[]=strtoupper($value{0}).substr($value,1);
    $prenom=implode("-",$pren);
    unset ($pren);
    $prenoms=explode("'",$prenom);
    foreach ($prenoms as $value)
      $pren[]=strtoupper($value{0}).substr($value,1);
    $prenom=implode("'",$pren);
    unset ($pren);
    $prenoms=explode(".",$prenom);
    foreach ($prenoms as $value)
      $pren[]=strtoupper($value{0}).substr($value,1);
    $prenom=implode(".",$pren);
    return $prenom;
  }

  function LevenshteinDistance ($str1, $str2)
  {
    // Differences entre deux chaines. Renvoie le nombre de caracteres d'ecart
    // entre les deux chaines, meme si elles sont de taille identique
    $strlen1 = strlen ($str1);
    $strlen2 = strlen ($str2);
    $max = max ($strlen1, $strlen2);

    $splitSize = 250;
    if ($max > $splitSize)
    {
      $lev = 0;
      for ($cont = 0 ; $cont < $max ; $cont += $splitSize)
      {
        if ($strlen1 <= $cont || $strlen2 <= $cont)
        {
          $lev = $lev/ ($max/min ($strlen1, $strlen2));
          break;
        }
        $lev += levenshtein (substr ($str1, $cont, $splitSize),
                             substr ($str2, $cont, $splitSize));
      }
    }
    else
    $lev = levenshtein ($str1, $str2);
    return $lev;
  }

  function HumanReadableValue ($val, $kilo=1000, $precision=0)
  {
    // Cette fonction renvoie une chaine correspondant a la valeur numerique
    // avec le multiplicateur (1000000 -> 1M).
    // Fonctionne avec les grandes valeurs (K,M,G,T,P...), mais aussi avec les
    // petites (m, u, n, p)
    // $kilo peut valoir 1024 pour travailler avec des octets
    // $precision est le nombre de chiffres apres la virgule
    $res = "";
    if ($val < 0)
    {
      $res.= "-";
      $val = abs ($val);
    }
    if ($val == 0)
     return $res.round ($val, $precision);
    elseif ($val < 1/$kilo/$kilo/$kilo/$kilo/$kilo/$kilo)
      return $res.round ($val*$kilo*$kilo*$kilo*$kilo*$kilo*$kilo*$kilo, $precision)."z";
    elseif ($val < 1/$kilo/$kilo/$kilo/$kilo/$kilo)
      return $res.round ($val*$kilo*$kilo*$kilo*$kilo*$kilo*$kilo, $precision)."a";
    elseif ($val < 1/$kilo/$kilo/$kilo/$kilo)
      return $res.round ($val*$kilo*$kilo*$kilo*$kilo*$kilo, $precision)."f";
    elseif ($val < 1/$kilo/$kilo/$kilo)
      return $res.round ($val*$kilo*$kilo*$kilo*$kilo, $precision)."p";
    elseif ($val < 1/$kilo/$kilo)
      return $res.round ($val*$kilo*$kilo*$kilo, $precision)."n";
    elseif ($val < 1/$kilo)
      return $res.round ($val*$kilo*$kilo, $precision)."u";
    elseif ($val < 1)
      return $res.round ($val*$kilo, $precision)."m";
    elseif ($val < $kilo)
      return $res.round ($val, $precision);
    elseif ($val < $kilo*$kilo)
      return $res.round ($val/$kilo, $precision)."K";
    elseif ($val < $kilo*$kilo*$kilo)
      return $res.round ($val/$kilo/$kilo, $precision)."M";
    elseif ($val < $kilo*$kilo*$kilo*$kilo)
      return $res.round ($val/$kilo/$kilo/$kilo, $precision)."G";
    elseif ($val < $kilo*$kilo*$kilo*$kilo*$kilo)
      return $res.round ($val/$kilo/$kilo/$kilo/$kilo, $precision)."T";
    elseif ($val < $kilo*$kilo*$kilo*$kilo*$kilo*$kilo)
      return $res.round ($val/$kilo/$kilo/$kilo/$kilo/$kilo, $precision)."P";
    elseif ($val < $kilo*$kilo*$kilo*$kilo*$kilo*$kilo*$kilo)
      return $res.round ($val/$kilo/$kilo/$kilo/$kilo/$kilo/$kilo, $precision)."E";
    elseif ($val < $kilo*$kilo*$kilo*$kilo*$kilo*$kilo*$kilo*$kilo)
      return $res.round ($val/$kilo/$kilo/$kilo/$kilo/$kilo/$kilo/$kilo, $precision)."Z";
    elseif ($val < $kilo*$kilo*$kilo*$kilo*$kilo*$kilo*$kilo*$kilo*$kilo)
      return $res.round ($val/$kilo/$kilo/$kilo/$kilo/$kilo/$kilo/$kilo/$kilo, $precision)."Y";
    // En cas d'erreur ou rien ne marche, renvoie la valeur betement.
    return $val;
  }
?>
