<?
  // $Id: debug.php 15 2005-11-07 22:37:29Z fournier $
  /**
   * @author Dominique Fournier <dominique@fournier.homedns.org>
   * @param $level Niveau de verbosité EN OCTAL (Plus c'est haut, plus c'est verbeux)
   * @param $chaine Message à écrire dans le log
   * @return void
   *
   * Ecrit dans le fichier '/tmp/debugphp.log' si debug est supérieur ou égal à level
   * Ex : debug ( 07, "Très verbeux", __FILE__, __LINE__ );
   * Ex : debug ( 01, "Information capitale", __FILE__, __LINE__ );
   * Une constante DEBUG DOIT être placée en constante pour activer cette fonction :
   *   define( "DEBUG", 01 ); // En octal
   * Cette variable aura pour valeur le niveau maximal à afficher :
   * Ex : DEBUG = 0; -> Aucun debug (ou niveau 0 uniquement)
   * Ex : DEBUG = 1; -> debug de niveau 0 + de niveau 1
   * Ex : DEBUG = 2; -> debug de niveau 0 + de niveau 1 + niveau 2
   *
   * ATTENTION : Le mode octal empeche plus de 7 niveaux de debug !!!
   *
   * Si on ne veut pas de ligne ni de fichier, il suffit de ne pas les mettre
   *
   * Version 1.0 : Passage en octal (compatible avec l'ancienne version)
   *               L'octal permet de coder plusieurs elements a debugger (chaque 
   *               chiffre code une partie a debugger)
   *               Modification des parametres pour avoir le nom de fichier et la ligne
   */
function debug ( $level, $chaine, $fichier="", $ligne="" )
{
  if ( ! defined('DEBUG') ) return ;

  // On va cherche un supmask qui vaudra 7 la ou il y a un chiffre dans level
  $mask = 07;
  $supmask = 0;
  for ( $i = 1 ; $i <= 15 ; $i=$i+3 ) // Nombre de bits autorises dans le debug
                                      // Doit etre un multiple de 3
  {
    if ( ( $level & $mask ) > 0 ) $supmask += $mask;
    $mask = $mask << 3;
  }

  // On vire ce que le debug ne veut pas
  if ( ( DEBUG & $supmask ) < $level ) return ;

  // Mise en page et enregistrement
  $level = sprintf ( "%o", $level );
  if ( $ligne != "" ) $ligne = " (ligne $ligne)";
  if ( $fichier != "" ) $fichier = " $fichier";
  $query="log=$fichier$ligne [debug $level] $chaine";
  $host = "localhost";
  $path = "/libphp/phpsyslog/";
  $post  = "POST $path HTTP/1.1\r\nHost: $host\r\n";
  $post .= "Content-type: application/x-www-form-urlencoded\r\n";
  $post .= "User-Agent: phpsyslog\r\nContent-length: ".strlen($query)."\r\n";
  $post .= "Connection: close\r\n\r\n$query";
  $h = fsockopen ( $host, 80 );
  fwrite ( $h, $post );
  //for ( $a=0, $r=''; !$a; )
  //{
  //  $b = fread ( $h, 8192 );
  //  $r .= $b;
  //  $a = ( ( $b == '' ) ? 1 : 0 );
  //}
  fclose ( $h );
}
?>
