<?
  // alerte.php

  // Connexion par HTTPS vers une machine distante pour envoyer des alertes
  // audio et logguer des informations

  error_reporting(E_ALL | E_STRICT);

  ////////////////////////////////
  // VERIFICATIONS DE DEMARRAGE //
  ////////////////////////////////
  if (!in_array ("https", stream_get_wrappers()))
    die ("Pas de support HTTPS activé pour le PHP\n");

  ////////////////////
  // CONFIRGURATION //
  ////////////////////

  ///////////////
  // FONCTIONS //
  ///////////////
  function alerte ($message, $audio="", $maxrepetition=0, $machine="", 
                   $urlAlerte="", $timeout=3, $mail=FALSE)
  {
    // Cette fonction envoie une connexion HTTPS vers un serveur defini dans la
    // configuration avec les informations contenues dans les parametres.
    // En cas de panne de la station de supervision, un mail est envoye a
    // l'administrateur defini dans la configuration.
    // $message : le message place dans les logs.
    // $audio : la serie de son joues
    // $maxrepetition : le nombre de minutes entre deux rejeux du meme message
    // $machine   : adresseIP:port de la machine de supervision (utiliser l'IP
    //              en cas de panne DNS)
    // $urlAlerte : doit commencer par https:// ou http:// et donner le php 
    //              alerteCluster.php
    // $timeout   : Temps maximum de connexion vers la station de supervision
    // $mail      : Mail de l'administrateur en cas de problème avec la machine
    // de supervision

    $url=parse_url ($urlAlerte);
    if ($url["scheme"] === "https")
      $protocole = "ssl";
    elseif ($url["scheme"] === "http")
      $protocole = "tcp";
    else
    {
      criclog ("alerte.php : schéma de l'URL de \$url inconnu");
      return;
    }
    unset ($erreur);
    unset ($erreurnet);
    $fp = @stream_socket_client ("$protocole://$machine", $errno, $errstr, 
                                  $timeout);
    if (!$fp)
    {
      // La machine de supervision est injoignable
      $erreurnet = $errstr;
      //echo "$errstr ($errno)\n";
    }
    else
    {
      // Si le serveur met plus de $timeout secondes a repondre, il y a un
      // probleme.
      stream_set_timeout ($fp, $timeout);

      $serveur=hostname(hostnameLocal());
      if ($message !== "test" && $message !== "demarrageMachine")
      {
        // Le message de test ne doit pas etre precede du serveur
        $message="[$serveur] $message";
      }
      if ($message === "demarrageMachine")
      {
        // Dans le cas du demarrage, on envoie le message 'demarrageMachine
        // $serveur' pour reinitialiser les compteurs de la station de
        // surveillance pour ce serveur
        $message = "demarrageMachine $serveur";
      }
      $commande = "message=".urlencode($message);
      if (isset ($audio) && $audio!="")
      {
        $audio = "$serveur $audio";
        $commande .= "&audio=".urlencode($audio);
      }
      if (isset ($maxrepetition) && $maxrepetition!=0)
        $commande .= "&maxrepetition=$maxrepetition";

      $headers  = "Host: ".$url["host"]."\r\n";
      $headers .= "User-Agent: Cluster\r\n";
      $headers .= "Content-Length: ".strlen($commande)."\r\n";
      $headers .= "Content-Type: application/x-www-form-urlencoded\r\n";

      $transaction  = "POST $url HTTP/1.0\r\n";
      $transaction .= $headers;
      $transaction .= "\r\n";
      $transaction .= $commande;
      // echo $transaction;
      fwrite ($fp, $transaction);
      $result=FALSE;
      // On veut le code de retour du serveur
      // Il est de la forme :
      // HTTP/1.1 404 Not Found
      $result=fgets ($fp, 1024);
      // On recupere  le code et le message
      list ($http, $code, $message) = explode (" ", $result);
      switch ($code)
      {
        case "404":
          $erreur="404 URL introuvable....\n";
          break;
        case "200":
           // Cool la connexion marche bien, regarde si on a eu une erreur
           break;
        default:
          // Un erreur est signalee, on va envoyer un mail
          $erreur=$result;
          break;
      }
      if (!isset($erreur))
      {
        // On saute les entetes du serveur (jusqu'a la ligne blanche)
        while (($ligne = fgets ($fp, 1024)) !== FALSE)
        {
          if (trim ($ligne) === "") break;
        }
        $result = "";
        while (($ligne = fgets ($fp, 1024)) !== FALSE)
        {
          $result.=$ligne;
        }
        // On verifie si l'arret est du a une fin de connexion a cause de
        // timeout
        $info = stream_get_meta_data ($fp);
        if ($info['timed_out'])
        {
          $erreurnet = "Connection timed out !";
        }
        if ($result !== "OK\n")
        {
          // Une erreur a ete renvoyee par le PHP de la supervision, on envoie
          // un mail
          $erreur = $result;
        }
      }
      fclose ($fp);
    }
    if (isset ($erreur))
    {
      // Erreur signalee lors de la connexion avec le serveur de surveillance
      // on envoie un mail a l'administrateur
      $sujet = "ALERTE : Problème cluster sur ".hostnameLocal();
      $message  = "Message du cluster de ".hostnameLocal()." :\r\n";
      $message .= "\r\n";
      $message .= "La machine de supervision '$machine";
      $message .= "' a renvoyé l'erreur suivante :\r\n";
      $message .= $erreur;
      criclog ("ERREUR : '$machine' : ".trim($erreur));
    }
    if (isset ($erreurnet))
    {
      // Erreur signalee lors de la connexion avec le serveur de surveillance
      // on envoie un mail a l'administrateur
      $sujet = "ALERTE : Problème cluster sur ".hostnameLocal();
      $message  = "Message du cluster de ".hostnameLocal()." :\r\n";
      $message .= "\r\n";
      $message .= "La machine de supervision '$machine";
      $message .= "' est injoignable : \r\n";
      $message .= $erreurnet;
      criclog ("ERREUR : '$machine' injoignable : ".
               trim($erreurnet));
    }
    if (isset ($erreur) || isset ($erreurnet))
    {
      // Une erreur reseau ou une erreur de fonctionnement a ete detectee, on
      // envoie le mail avec le message et le sujet definis ci-dessus, sauf si
      // le meme mail a deja ete envoye dans les 30 dernieres minutes.
      $fichierBloqueMail = "run/".urlencode($sujet);
      clearstatcache();
      if (!file_exists ($fichierBloqueMail) ||
          filemtime ($fichierBloqueMail)+60*30 < time ())
      {
        touch ($fichierBloqueMail);
        $headers='From: CLUSTER@'.hostnameLocal() . "\r\n" .
                 "Reply-To: $mail\r\n" .
                 'X-Mailer: PHP/' . phpversion()."\r\n".
                 'Content-Type: text/plain; charset=utf-8';
        /*
        echo "*********** MAIL ************\n";
        echo $headers."\n";
        echo $sujet."\n".$message;
        echo "*****************************\n";
        return FALSE;
        */
        if (smtp_mail ($mail, $sujet, $message, $headers)===
            FALSE)
        {
          criclog ("ERREUR : Le mail d'information n'est pas parti");
        }
      }
      return FALSE;
    }
    return $result;
  }

  ///////////
  // TESTS //
  ///////////
  /*
  echo alerte ("test");
  echo alerte ("Message de test 1");
  echo alerte ("Message de test 2", "coucou behh");
  echo alerte ("Message de test 3", "coucou", 2)
  echo alerte ("demarrageMachine");
  */
?>
